function [data_rec, dsdr_rec, obj_func, cnt] = chambolle_pock(data_quantized, param, data)
% CHAMBOLLE_POCK is the implementation of Chamboll-Pock algorithm suited for
% the needs of signal dequantization

% setting different parameter for the simulation
paramsolver.maxit = 1000;    % maximum number of iterations
paramsolver.minit = 50;     % minimum number of iterations


% CP parameter; here step for projection
% predefined_zetas are empirical values of zetas for different frametype and level of quantization
switch param.frametype
    case 'dgtreal'
        predefined_zetas = [0.0055; 0.0031; 0.0013; 0.00017; 0.000041; 0.000015; 0.0000057];
        paramsolver.zeta = predefined_zetas(param.wordlength - 1); 
    case 'wmdct'
        predefined_zetas = [0.0213; 0.0110; 0.0053; 0.00023; 0.000066; 0.000022; 0.0000075];
        paramsolver.zeta = predefined_zetas(param.wordlength - 1);          
    otherwise
        error('Invalid frametype!')
end

paramsolver.sigma = 1/paramsolver.zeta;     % CP parameter; here threshold for soft thresholding
paramsolver.rho = 1;    % step size for CP algorithm rho = <0,1>

paramsolver.verbose = 0;      % display parameter
paramsolver.comp_dsdr = 1;    % compute and store dSDR during iterations
paramsolver.dsdr_decrease_termination = 1;  % terminate algorithm if the SDR value starts to decrease
paramsolver.comp_obj = 1;     % compute and store objective function values during iterations

% starting poing
sol.primal = data_quantized; 
sol.dual = frana(param.F, sol.primal);

data_rec = data_quantized; % initialization of reconstructed data vector

% definition of clip function (result of the Fenchel-Rockafellar conjugate of soft thresholding)
clip = @(x) (sign(x).*min(abs(x), 1));

% dsdr process
dsdr_rec = NaN(paramsolver.maxit, 1);

% objective function process
obj_func = NaN(paramsolver.maxit, 1);

% iteration counter
cnt = 0;

while cnt <= paramsolver.maxit
    cnt = cnt+1;
    
    sol.dual = clip(sol.dual + paramsolver.sigma.*frana(param.F, data_rec));
    
    sol.primal_old = sol.primal;
    sol.primal = proj_time(sol.primal - paramsolver.zeta*postpad(frsyn(param.F, sol.dual), param.Ls), data_quantized, param.delta);
    data_rec = sol.primal + paramsolver.rho*(sol.primal - sol.primal_old);
    %data_rec = proj_time(data_rec, data_quant, param.delta);
        
    if paramsolver.comp_obj
        obj_func(cnt) = norm(frana(param.F, data_rec), 1); % computing objective function (l1 norm of coefficients)
    end
    
    if paramsolver.comp_dsdr || paramsolver.dsdr_decrease_termination
        dsdr_rec(cnt) = sdr(data, data_rec) - sdr(data, data_quantized); % computing dSDR
        if paramsolver.dsdr_decrease_termination && cnt > paramsolver.minit && dsdr_rec(cnt) - dsdr_rec(cnt-1) < 0
            break
        end
    end
    
    if paramsolver.verbose
        fprintf(' Iteration number: %u\n', cnt);
        if paramsolver.comp_dsdr
            fprintf(' SDR improvement: %e \n', dsdr_rec(cnt));
        end
        if paramsolver.comp_obj
            fprintf(' Objective function value: %e \n', obj_func(cnt));
        end
        fprintf('\n')        
    end
       
end

data_rec = proj_time(data_rec, data_quantized, param.delta); % final projection into the constraints

end
