%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%                                %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%    DEQUANTIZATION MAIN FILE    %%%%%%%%%%%%%%%%%%%%%%%     
%%%%%%%%%%%%%%%%%%%%                                %%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%% input file settings
% audio file path
fprintf('Loading audio\n')
%audio_path = 'Speech_IEEE_corpus_narrowband'; % 8 kHz
audio_path = 'Speech_IEEE_corpus_wideband'; % 25 kHz

audio_file = 'S_01_01'; % S_XX_YY, XX \in <01, 75>, Y \in <1,10>
         
% load audio-file
[data, fs] = audioread([audio_path '\' audio_file '.wav']);

% downsampling to 16 kHz
fprintf('Downsampling audio to 16 kHz\n')
fs_new = 16000;
data = resample(data, fs_new, fs);

% peak-normalization
fprintf('Normalizing audio...')
maxAbsVal = max(abs(data));
data = data/maxAbsVal;
fprintf('(The original maximum was %f)\n', maxAbsVal)

% signal length
param.Ls = length(data);

%% settings
fprintf('Setting up the frame parameters\n')
param.wordlength = 5;     % set the wordlength in bits
param.frametype = 'dgtreal';   % set type of frame used for reconstruction, options: 'dgtreal', 'wmdct'.

switch param.frametype
    case 'dgtreal' % real valued Discrete Gabor Transform
        param.w = 1024;
        param.a = param.w/4;
        param.M = 1024; % M >= w
        param.wtype = 'hann'; % options available on: http://ltfat.github.io/doc/sigproc/firwin.html
    case 'wmdct' % Windowed Modified Discrete Cosine Transform
        param.M = 1024;
        param.wtype = 'hann';
end

param.algorithm = 'DR'; % algorithm to compute declipping, options: 'Douglas-Rachford', 'Chamboll-Pock'
fprintf('Setting up algorithm: %s\n', param.algorithm)


%% quantization 
fprintf('Generating quantized signal\n')
[data_quant, param.delta] = quant(data, param.wordlength); % quantizing the original signal and computing the quantization step


%% construction of frame
fprintf('Creating the frame\n')

switch param.frametype
    case 'dgtreal'
        param.F = frametight(frame('dgtreal', {param.wtype, param.w}, param.a, param.M));
    case 'wmdct'
        param.F = frametight(frame('wmdct', param.wtype, param.M));
    otherwise 
        error('Invalid frametype!');
end

param.F = frameaccel(param.F, param.Ls);  % precomputation for a fixed signal length

%% Proximal algorithm
fprintf('Starting the proximal algorithm\n')

tic; 

    switch param.algorithm
        case {'Douglas-Rachford', 'DR'} % synthesis model of the signal          
            [data_rec, dsdr_iter, obj_iter, iterations] = douglas_rachford(data_quant, param, data);
            
        case {'Chamboll-Pock', 'CP'} % analysis model of the signal
            [data_rec, dsdr_iter, obj_iter, iterations] = chambolle_pock(data_quant, param, data);
        otherwise
            error('Invalid algorithm is set!');
    end
    
time = toc;


%% Plot dequantized signal
fprintf('Computing final SDR and plotting some nice figures\n')

% time
fprintf('Result obtained in %4.3f seconds.\n', time);

% SDR
sdr_quant = sdr(data, data_quant);
sdr_rec = sdr(data, data_rec);
dsdr = sdr_rec - sdr_quant;
fprintf('SDR of the quantized signal is %4.3f dB.\n', sdr_quant);
fprintf('SDR of the reconstructed signal is %4.3f dB.\n', sdr_rec);
fprintf('SDR improvement is %4.3f dB.\n', dsdr);

% Plot of signal waveforms
t = linspace(0, length(data)/fs, length(data));
figure
subplot(3,1,1);
plot(t, data);
xlabel('time (s)')
ylabel('Amplitude')
title('Original')
subplot(3,1,2);
plot(t, data_quant);
xlabel('time (s)')
ylabel('Amplitude')
title('Quantized')
subplot(3,1,3);
plot(t, data_rec);
xlabel('time (s)')
ylabel('Amplitude')
title('Restored')

% Plot signal spectrograms
figure
subplot(1,3,1)
sgram(data, 'fs', fs_new, 'dynrange', 80);
title('Original')
subplot(1,3,2)
sgram(data_quant, 'fs', fs_new, 'dynrange', 80);
title('Quantized')
subplot(1,3,3)
sgram(data_rec, 'fs', fs_new, 'dynrange', 80);
title('Restored')

% Plot of dSDR and objective function
figure
len = length(dsdr_iter(~isnan(dsdr_iter)));
t = linspace(0, time, len);
yyaxis left
    p1 = plot(t.',dsdr_iter(~isnan(dsdr_iter)));
    hold on
    ylabel('{\Delta}SDR (dB)');
      
yyaxis right
    p2 = plot(t.', obj_iter(~isnan(obj_iter)));
    xlabel('time (s)');
    ylabel('Objective function');
    grid on
