function [data_rec, dsdr_rec, obj_func, cnt] = douglas_rachford(data_quantized, param, data)
% DOUGLAS_RACHFORD is the implementation of Douglas-Rachford algorithm suited
% for the needs of signal dequantization

% setting different parameter for the simulation
paramsolver.maxit = 1000;    % maximum number of iterations
paramsolver.minit = 50;     % minimum number of iterations 
paramsolver.lambda = 1;      % step size for DR algorithm (default 1)

% DR parameter; here the threshold for soft thresholding
% predefined_gammas are empirical values of gammas for different frametype and level of quantization
switch param.frametype
    case 'dgtreal'
        predefined_gammas = [0.0073; 0.0040; 0.0015; 0.00025; 0.000049; 0.000017; 0.0000066];
        paramsolver.gamma = predefined_gammas(param.wordlength - 1); 
    case 'wmdct'
        predefined_gammas = [0.0204; 0.0123; 0.0055; 0.00035; 0.000084; 0.000028; 0.0000099];
        paramsolver.gamma = predefined_gammas(param.wordlength - 1);          
    otherwise
        error('Invalid frametype!')
end
    
paramsolver.verbose = 0;      % display parameter
paramsolver.comp_dsdr = 1;    % compute and store dSDR during iterations
paramsolver.dsdr_decrease_termination = 1;  % terminate algorithm if the SDR value starts to decrease
paramsolver.comp_obj = 1;     % compute and store objective function values during iterations

% starting point
c = frana(param.F, data_quantized);
% c = zeros(length(c), 1);

% definition of soft thresholding
soft = @(z, T) sign(z).*max(abs(z)-T, 0);

% dsdr process
dsdr_rec = NaN(paramsolver.maxit, 1);

% objective function process
obj_func = NaN(paramsolver.maxit, 1);

% iteration counter
cnt = 0;

while cnt <= paramsolver.maxit
    cnt = cnt + 1;
    
    c_tilde = proj_parse_frame(c, param.F, data_quantized, param.delta);
    c = c + paramsolver.lambda*(soft(2*c_tilde-c, paramsolver.gamma)-c_tilde);
    
    if paramsolver.comp_obj
        obj_func(cnt) = norm(c_tilde, 1); % computing objective function (l1 norm of coefficients)
    end
    
    if paramsolver.comp_dsdr || paramsolver.dsdr_decrease_termination
        data_rec_tmp = postpad(frsyn(param.F, c_tilde), param.Ls); % reconstructed signal
        dsdr_rec(cnt) = sdr(data, data_rec_tmp) - sdr(data, data_quantized); % computing dSDR
        if paramsolver.dsdr_decrease_termination && cnt > paramsolver.minit && dsdr_rec(cnt) - dsdr_rec(cnt-1) < 0
            break
        end
    end
       
    if paramsolver.verbose
        fprintf(' Iteration number: %u\n', cnt);
        if paramsolver.comp_dsdr
            fprintf(' SDR improvement: %e \n', dsdr_rec(cnt));
        end
        if paramsolver.comp_obj
            fprintf(' Objective function value: %e \n', obj_func(cnt));
        end
        fprintf('\n')
    end
    
end


c_tilde = proj_parse_frame(c, param.F, data_quantized, param.delta); % final projection into the constraints
data_rec = postpad(frsyn(param.F, c_tilde), param.Ls); % reconstructed signal

end
